from rpy2.robjects.vectors import FloatVector
from rpy2.robjects.packages import importr
import rpy2.rinterface as ri
#stats = importr('stats')


__all__ = ["pyfunc2R", "rosen", "test1", "rosen_for_r", "test1_for_r"]


def pyfunc2R(f):
    """Converts a python function into a R object

    ::

        def f(x,y):
            return x+y

        f4r = pyfunc2R(f)
    """
    fr = ri.rternalize(f)
    return fr

# Rosenbrock Banana function as a cost function
# (as in the R man page for optim())
def rosen(x):
    """Rosenbrock Banana function as a cost function
     (as in the R man page for optim())
    """
    x1 = x[0]
    x2 = x[1]
    return 100 * (x2 - x1 * x1)**2 + (1 - x1)**2

#: wrap the function rosen so it can be exposed to R
rosen_for_r = pyfunc2R(rosen)

def test1(x):
    r"""A simple example. 


    .. math::

        4x^2-2.1x^4 + 1/3 * x^6+xy-4y^2+4y^4

    """
    x1 = x[0]
    x2 = x[1]
    y  = 4*x1*x1-2.1*x1**4 + 1./3.*x1**6+x1*x2-4*x2*x2+4*x2**4;

    return y

#: wrap the function test1 so it can be exposed to R
test1_for_r = pyfunc2R(test1)




from pymeigo import funcs
from rpy2.robjects import FloatVector

def test_func_rosen():
    a = funcs.rosen([1,2.])
    b = funcs.rosen_for_r(FloatVector([1,2]))
    assert a ==  b[0]


def test_test1():
    a = funcs.test1([1,1])



from pymeigo import essR, rosen_for_r, test1_for_r
#from pymeigo import *

def test_rosen():
    essR(f=rosen_for_r, x_L=[-1,-1], x_U=[2,2])

def _test_test1():
    essR(f=test1_for_r, x_L=[-1,-1], x_U=[2,2])



# -*- python -*-
#
#  This file is part of the pymeigo software
#
#  Copyright (c) 2011-2012 - EBI, EMBL
#
#  File author(s): (cokelaer@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  pymeigo: http://www.ebi.ac.uk/~cokelaer/pymeigo
#
##############################################################################
"""Provide a Python interface to MEIGO (a R package).

::

    >>> pymeigo import essR, rosen_for_r
    >>> data = essR(f=rosen_for_r, x_L=[-1,-1], x_U=[2,2,])


The full documentation of the essR function comes from the R package itself.


The are two sets of parameters. See :func:`set_problem_options` and
:func:`ssm_default`.


"""
__author__ = """\n""".join(['Thomas Cokelaer <cokelaer@ebi.ac.uk'])
__revision__ = "$Rev: 2065 $"

# Use Rnames2attributes decorator to ease the access to R names
from cnolab.wrapper.tools import *

__all__  = ['essR', 'ssm_default', 'set_problem_options']

import os
from rpy2 import robjects
from rpy2.robjects.packages import importr
rpack_MEIGOR = load_Rpackage("MEIGOR")
import rpy2.rinterface



def Rsetdoc(f):
    """Decorator that copy the R doc into the wrapped Python function.

    .. note:: function to be used for developers only as a decorator to
       R function.

    """
    name = f.__name__
    doc = buildDocString("MEIGOR", name)
    if f.__doc__:
        f.__doc__ += doc
    else:
        f.__doc__ = doc
    return f




def set_problem_options(f=None, x_L=[], x_U=[], x_0=None, f_0=None, neq=None, c_L=None,
c_U=None, int_var=None, bin_var=None, vtr=None, **kargs):
    """INPUT PARAMETERS:

    :param f: Name of the file containing the objective  function (String)
    :param x_L: Lower bounds of decision variables (vector)
    :param x_U: Upper bounds of decision variables (vector)
    :param x_0: Initial point(s) (optional; vector or matrix)
    :param f_0: Function values of initial point(s) (optional)

    .. note:: The dimension of f_0 and x_0 may be different. For example, if
        we want to introduce 5 initial points but we only know the values
        for 3 of them, x_0 would have 5 rows whereas f_0 would have only 3
        elements. In this example, it is mandatory that the first 3 
        rows of x_0 correspond  to the values of f_0
      
    Additionally, fill the following fields if your problem has non-linear constraints
     
    :param neq: Number of equality constraints (Integer; do not define it if there are no equality constraints)
    :param c_L: Lower bounds of nonlinear inequality constraints (vector)
    :param c_U: Upper bounds of nonlinear inequality constraints (vector)
    :param int_var: Number of integer variables (Integer)
    :param bin_var: Number of binary variables  (Integer)
    :param vtr: Objective function value to be reached opional)
    """


    p = {}

    if f:
        p['f'] = f
    else:
        raise ArgError("f must be provided.")
    p['x_L'] = robjects.FloatVector(x_L)
    p['x_U'] = robjects.FloatVector(x_U)

    if x_0:
        p['x_0'] = x_0

    if f_0:
        p['f_0'] = f_0

    if neq:
        p['neq'] = neq

    if c_L: p['c_L']= c_L
    if c_U: p['c_U'] = c_U
    if int_var: p['int_var'] = int_var
    if bin_var: p['bin_var'] = bin_var
    if vtr: p['vtr'] = vtr
    return robjects.ListVector(p)

def ssm_default():
    """Return the default parameters required by the SSM algorithm.

     List containing options (if set as opts <- numeric(0) default options
     will be loaded). See the script of default options to know their values
  
     :User options:

       * maxeval = Maximum number of function evaluations  (Default 1000)
       * maxtime = Maximum CPU time in seconds (Default 60)
       * iterprint  = Print each iteration on screen: 0-Deactivated 1-Activated (Default 1)
       * weight     = Weight that multiplies the penalty term added to the objective function in constrained  problems (Default 1e6)
       * log_var    = Indexes of the variables which will be used to generate diverse solutions in different  orders of magnitude (vector)
       * tolc       = Maximum absolute violation of theconstraints  (Default 1e-5)
       * prob_bound = Probability (0-1) of biasing the search towards  the bounds (Default 0.5)
       * inter_save = Saves results of intermediate iterations in eSSR_report.RData Useful for very long runs. (Binary; Default = 0)

     :Global options:

       * dim_refset: Number of elements in Refset (Integer; automatically calculated)
       * ndiverse:  Number of solutions generated by the diversificator (Default 10*nvar)
       * combination: Type of combination of Refset elements (Default 1)
            #. hyper-rectangles
            #. linear combinations

    :Local options:
       *  local_solver: Choose local solver  0: Local search deactivated (Default), "NM", "BFGS", "CG", "LBFGSB", "SA","SOLNP", "DHC", "NLS2SOL"
       * local_tol: Level of tolerance in local search
       * local_iterprint: print each iteration of local solver on screen (Binary; default = 0)
       * local_n1: Number of iterations before applying local search for the 1st time (Default 1)
       * local_n2: Minimum number of iterations in the global phase between 2 local calls (Default 10)
       * local_balance: Balances between quality (=0) and diversity (=1) for choosing initial points for  the local search (default 0.5)
       * local_finish: Applies local search to the best solution found once the optimization if finished (same values as opts.local.solver)
       * local_bestx: When activated (i.e. =1) only applies local search to the best solution found to date,ignoring filters (Default=0)
    """
    return rpack_MEIGOR.ssm_defaults()

def _get_opts(**kargs):
    """scan all ssm_Default values"""
    defaults = ssm_default()
    o = {}
    for x in defaults.names:
        o[x] = kargs.get(x, defaults.rx2(x)) # 1000
    print o
    return robjects.ListVector(o)



@Rsetdoc
@Rnames2attributes
def essR(**kargs):
    problem = set_problem_options(**kargs)
    opts = _get_opts(**kargs)
    """
    problem<-list(f="ex1",x_L=rep(-1,2),x_U=rep(1,2))
    opts<-list(maxeval=500, ndiverse=10, dim_refset=4, local_solver="solnp",
    local_n2=1)
    #========================= END OF PROBLEM SPECIFICATIONS =====================
    Results<-essR(problem,opts);
    """
    return rpack_MEIGOR.essR(problem, opts)

essR.__doc__ = """This is an automatic scan of the R manual. Layout may not be
perfect...Please have a look at :func:`ssm_default` and
:func:`set_problem_options`.\n\n""" + essR.__doc__




# -* python -*-
#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EBI
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv2 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-2.0.html
#
#  CNO website: http://www.ebi.ac.uk/saezrodriguez/software.html
#
##############################################################################
# $Id: meigo.py 2060 2012-08-12 17:59:38Z cokelaer $
"""Provide a class to run MEIGO optiisation


"""
import os
import copy
import time
from os.path import join as pj

from rpy2 import robjects

from wrapper_meigo import essR
from pylab import *
__all__ = ["MEIGO"]



class MEIGO(object):
    """class MEIGO around :mod:`pymeigo.wrapper_meigo`


    .. plot::
        :include-source:

        from pymeigo import MEIGO, rosen_for_r
        m = MEIGO(f=rosen_for_r)
        m.run(x_U=[2,2], x_L=[-1,-1])
        m.plot()

    """
    def __init__(self, f):
        """
        :param f: a R version of a python function. See :mod:`~pymeigo.funcs.pyfunc2R`.
        """
        self.func = f

    def run(self, x_U=[2,2], x_L=[-1,-1], **kargs):
        """


        :param x_U: upper limit of the parameter
        :param x_U: lower limit of the parameter

        see :func:`~pymeigo.wrapper_meigo.essR` for arguments.

        """
        self.res =  essR(f=self.func, x_U=x_U, x_L=x_L, **kargs)

    def plot(self):
        """plotting the evolution of the cost function"""

        plot(self.res.f)
        xlabel("Evaluation")
        ylabel("Cost function")


        semilogy(self.res.f)
        xlabel("Evaluation")



# -*- coding: utf-8 -*-
#
# documentation build configuration file, created by
# sphinx-quickstart on Wed Aug 10 16:58:13 2011.
#
# This file is execfile()d with the current directory set to its containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import sys, os

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.

pkg_name = "pymeigo"

try:
    exec("import %s" % pkg_name)
except Exception, e:
    print "Install %s or set your PYTHONPATH properly" % pkg_name
    raise Exception

try:
    import cnolab.deploy
    from cnolab.deploy import get_path_sphinx_themes
except Exception, e:
    print "Install cnolab.deploy or set your PYTHONPATH properly"
    raise Exception


import pkg_resources
version = pkg_resources.require(pkg_name)[0].version
release = version
author = "Thomas Cokelaer"
title = pkg_name
copyright = author + ", 2012"
project = pkg_name




# -- General configuration -----------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.

extensions = [
    'sphinx.ext.autodoc', 
    'sphinx.ext.autosummary', 
    'sphinx.ext.coverage', 
    'sphinx.ext.graphviz',
    'sphinx.ext.doctest', 
    'sphinx.ext.intersphinx', 
    'sphinx.ext.todo',
    'sphinx.ext.coverage', 
    'sphinx.ext.pngmath',
    'sphinx.ext.ifconfig',
    'sphinx.ext.viewcode',
    'cnolab.deploy.copybutton',
    'matplotlib.sphinxext.plot_directive',
    'matplotlib.sphinxext.only_directives',
    ]

todo_include_todos=True
jscopybutton_path = cnolab.deploy.copybutton.get_copybutton_path()
autoclass_content = 'both'

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The encoding of source files.
#source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = project
copyright = copyright

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = version
# The full version, including alpha/beta/rc tags.
release = release

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
#today_fmt = '%B %d, %Y'

# List of documents that shouldn't be included in the build.
#unused_docs = []


# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = []

# The reST default role (used for this markup: `text`) to use for all documents.
#default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
#add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = False

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
show_authors = True

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# A list of ignored prefixes for module index sorting.
#modindex_common_prefix = []


# -- Options for HTML output ---------------------------------------------------

# The theme to use for HTML and HTML Help pages.  Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
html_theme = 'standard'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
# the user theme contains the otpions 'homepage', which is populated here
#html_theme_options = {'homepage': init_sphinx.url}

# Add any paths that contain custom themes here, relative to this directory.
html_theme_path = [get_path_sphinx_themes()]



# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
#html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
#html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
#html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
#html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".

# the copybutton.js must be copied there: 
#html_static_path = ['_static']

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
html_index = 'index.html'

#Custom sidebar templates, maps page names to templates.
html_sidebars = {
                    'index': [ 'indexsidebar.html'], 
                    'contents':'indexsidebar.html',
}
# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {   'index': 'index.html'}


# If false, no module index is generated.
html_use_modindex = True

# If false, no index is generated.
html_use_index = True

# If true, the index is split into individual pages for each letter.
html_split_index = False

# If true, links to the reST sources are added to the pages.
#html_show_sourcelink = True
#html_copy_source = False

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
#html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
#html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = None

# Output file base name for HTML help builder.
htmlhelp_basename = 'doc'


# -- Options for LaTeX output --------------------------------------------------

# NOT in original quickstart
pngmath_use_preview = True

# The paper size ('letter' or 'a4').
latex_paper_size = 'a4'

# The font size ('10pt', '11pt' or '12pt').
latex_font_size = '10pt'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass [howto/manual]).
latex_documents = [
  ('index', 'main.tex', title,
   author, 'manual'),
]

latex_elements = { 'inputenc': '\\usepackage[utf8]{inputenc}' }

# The name of an image file (relative to this directory) to place at the top of
# the title page.
#latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
#latex_use_parts = False

# If true, show page references after internal links.
#latex_show_pagerefs = False

# If true, show URL addresses after external links.
#latex_show_urls = False

# Additional stuff for the LaTeX preamble.
latex_preamble =r"""
\definecolor{VerbatimColor}{rgb}{.9,1,0.9}
\definecolor{VerbatimBorderColor}{rgb}{0,0,0}

   \setlength{\fboxrule}{2pt}
 
 \renewcommand{\Verbatim}[1][1]{%
   % list starts new par, but we don't want it to be set apart vertically
   \bgroup\parskip=0pt%
   \smallskip%
   % The list environement is needed to control perfectly the vertical
   % space.
   \list{}{%
   \setlength\parskip{5pt}% space between verbatim and previous paragraph
   \setlength\itemsep{0ex}%
   \setlength\topsep{1ex}%
   \setlength\partopsep{0pt}%
   \setlength\leftmargin{10pt}%
   }%
   \item\MakeFramed {\FrameRestore}%
      \small%
     \OriginalVerbatim[#1]%
 }

"""

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
#latex_domain_indices = True


# -- Options for manual page output --------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('index', project, project,
     [author], 1)
]


# Example configuration for intersphinx: refer to the Python standard library.
#intersphinx_mapping = {'http://docs.python.org/': None}
intersphinx_mapping = {}



import rpy2
from rpy2 import robjects


def test_R_function():
    robjects.r('''
        f <- function(r, verbose=FALSE) {
            if (verbose) {
                cat("I am calling f().\n")
            }
            2 * pi * r
        }
        f(3)
        ''')
    r_f = robjects.r['f']

    assert r_f(3)[0]>18.84 and r_f(3)[0]<18.85  # should be a better test, but this is
                                                # just to demonstrate the usage or R
                                                # code inside ryp2 and python

    #<FloatVector - Python:0x5742290 / R:0x571d668>
    #[18.849556]




import pkg_resources
try:
    version = pkg_resources.require("pymeigo")[0].version
except:
    version = ""

import logging
logging.basicConfig(level=logging.ERROR)

logging.info("Importing pymeigo %s." % version)
logging.info("==========================")
logging.info("Checking that R packages can be loaded properly.")

from rpy2.robjects import rinterface
from rpy2.robjects import r


from cnolab.wrapper import tools
from cnolab.wrapper.tools import *

# set warning off
Rwarning(False)
try:
    import wrapper_meigo
    from wrapper_meigo import *
    logging.info(" * MEIGOR")
except ImportError,e:
    logging.warning(" * MEIGOR wrapper NOT loaded")
except rinterface.RRuntimeError, e:
    logging.warning(" * MEIGOR wrapper NOT loaded")
Rwarning(True)

import funcs
from funcs import *

import meigo
from meigo import *
